from Tkinter import *
import pylab
import time
import signal
import sys
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

def __sigint_handler__(signal, frame):
    print 'Exiting...'
    sys.exit(0)

class DOSIApplication(Frame):
    def plot_callback(self):
        a = self.txt.get()
        a = a.split()
        h = [float(i) for i in a]
        self.axes1.clear()
        self.axes2.clear()
        self.axes2.plot(pylab.np.abs(pylab.fft(h)))
        self.axes1.plot(h)
        self.axes1.set_title("Time Domain")
        self.axes2.set_title("FFT Magnitude Plot")

        self.canvas.show()

    def clear_plot(self):
        self.axes1.clear()
        self.axes2.clear()
        self.canvas.show()
        
    def __done_handler__(self):
        self.status_str.set("Ready")
        self.status_l.config(fg='green')

    def __run_handler__(self, event=None):
        self.status_str.set("Running")
        self.status_l.config(fg='yellow')
        self.after(2000,self.__done_handler__)


    def __save_handler__(self, event=None):
        self.status_str.set("Saving")
        self.status_l.config(fg='yellow')
        self.after(2000,self.__done_handler__)

    def __quit_handler__(self, event=None):
        self.quit()

    def __train_handler__(self, event=None):
        self.status_str.set("Ready")
        self.status_l.config(fg= 'green')


    def createWidgets(self):
        # Low frequency input label
        self.low_freq_l = Label(self, text="Low Frequency (MHz)")
        self.low_freq_l.grid(row=1, column=1)
        #Low frequency input box
        self.low_freq = Entry(self)
        self.low_freq.insert(0,"50")
        self.low_freq.grid(row=2, column=1)

        # High frequency input label
        self.high_freq_l = Label(self, text="High Frequency (MHz)")
        self.high_freq_l.grid(row=1, column=2)
        # High frequency input box
        self.high_freq = Entry(self)
        self.high_freq.insert(0,"400")
        self.high_freq.grid(row=2, column=2)

         # Step frequency Input Label
        self.step_freq_l = Label(self, text="Stop Frequency (MHz)")
        self.step_freq_l.grid(row=1, column=3)
        # Step frequency Input Box
        self.step_freq = Entry(self)
        self.step_freq.insert(0,"10")
        self.step_freq.grid(row=2, column=3)

        self.run_btn = Button(self, text="Train", command=self.__train_handler__, underline = 0)
        self.run_btn.grid(row=2, column=4)

        # Setup Time Domain and Fourier Transform Plots
        self.fig = pylab.Figure()
        self.axes1 = self.fig.add_subplot(2,1,1)
        self.axes2 = self.fig.add_subplot(2,1,2)
        self.axes1.set_title("Time Domain")
        self.axes2.set_title("FFT Magnitude Plot")

        self.canvas = FigureCanvasTkAgg(self.fig,master=self)
        self.canvas.show()
        self.canvas.get_tk_widget().grid(row=3, column=1, columnspan=5)

        self.run_btn = Button(self, text="Run Sweep", command=self.__run_handler__, underline = 0)
        self.run_btn.grid(row=4, column=1)

        self.save_btn = Button(self, text="Save Data", command=self.__save_handler__, underline=0)

        self.save_btn.grid(row=4,column=2)
        self.status_str = StringVar()
        self.status_str.set("Needs Training")
        self.status_l = Label(self, textvar=self.status_str, fg="yellow")
        self.status_l.grid(row=4,column=3)
        
        self.quit_btn = Button(self, text="Quit", command=self.quit, underline=0)
        self.quit_btn.grid(row=4, column=4)


    def key_handler(self, event=None):
        print 'pressed', repr(event.char)

        
    def __init__(self, master=None):
        Frame.__init__(self, master)
        master.bind('<Control-r>', self.__run_handler__)
        master.bind('<Control-s>', self.__save_handler__)
        master.bind('<Control-q>', self.__quit_handler__)

        self.pack()
        self.createWidgets()

def __delete_handler__(event=''):
    root.quit()

if __name__ == '__main__':
    signal.signal(signal.SIGINT, __sigint_handler__)
    root = Tk()
    root.bind('<Control-c>', __delete_handler__)
    root.protocol("WM_DELETE_WINDOW", __delete_handler__)
    app = DOSIApplication(master=root)
    app.mainloop()
    root.destroy()
