## 
# DOSITool.py: Commandline interface to DOSI
#
#
# Author : Chris Woodall < cwoodall at bu.edu >
# Date :  2013-08-08
## 

# Import everything we need
import time
import sys
import uhal
from dosi.hardware import *
from dosi.utils import *

__VERSION__ = "0.2"
__HELP_STR__ = """Dosi Tool version {0}

q - quits
h - displays this message
s(h|l|s|f) <value> [<units> optional] - set (h)igh, (l)ow and (s)tep 
                                        frequencies for a sweep (run with `r').
                                        Can also set an immediate (f)requency,
                                        which will take effect after the command
                                        is sent. <value> will be the value sent,
                                        it will be interpretted as <units> 
                                        dictates. Valid values for <units> are:
                                            - 'x': hex
                                            - 'i': integer
                                            - '(k|M)Hz': Converts from frequency
                                              to a FTW. (M)Hz and (k)Hz as well
                                              as just plain Hz is available.
r - run software sweep
""".format(__VERSION__)

import pylab
__mode__ = "debug"
if __mode__ == "debug":
    uhal.disableLogging()
    dosi = Dosi("/home/cwoodall/dosi-software/connections/dosi_connections.xml")

    # General Reset
    dosi.u2.device.getNode("ControlStatus").write(1)
    dosi.u2.device.dispatch()
    dosi.u2.device.getNode("ControlStatus").write(0x80)
    dosi.u2.device.dispatch()

#    raw_input("")

    # Set Training Mode
#    dosi.u2.device.getNode("ControlStatus").write(0x1000)
#    dosi.u2.device.dispatch()

    reg = 0
    while not (int(reg) & (1<<16)):
        reg = dosi.u2.device.getNode("PatternStatusRegister").read()
        dosi.u2.device.dispatch()
        print "fail"
    print hex(reg)
    print "TRAINED"
    raw_input("")
    dosi.u2.device.getNode("ControlStatus").write(1<<2)
    dosi.u2.device.dispatch()
   

    dosi.u2.device.getNode("ControlStatus").write(1<<13)
    dosi.u2.device.dispatch()

    time.sleep(.1)
    
    reg = dosi.u2.device.getNode("DI_SAMPLES").readBlock(512)
    dosi.u2.device.dispatch()
    
    for i in reg:
        print "0x{0:04x}".format(i)
 
    pylab.subplot(2,1,1)
    pylab.plot(reg)

    reg = dosi.u2.device.getNode("DQ_SAMPLE1").readBlock(512)
    dosi.u2.device.dispatch()
    for i in reg:
        print "0x{0:04x}".format(i)
 
    pylab.subplot(2,1,2)
    pylab.plot(reg)
    pylab.show()

elif __name__ == "__main__":
    dosi = Dosi("/home/cwoodall/dosi-software/connections/dosi_connections.xml")
    uhal.disableLogging()

    reg = dosi.u1.device.getNode("ControlStatus").read()
    dosi.u1.device.dispatch()
    
    dosi.u1.device.getNode("DDS1Output").write(1<<22)
    dosi.u1.device.dispatch()
    
    # Start Internal I/O update
    dosi.u1.spi.send([0x01, 0x00, 0xC0, 0x08, 0x20])
    # SDIO into an input (SDI) and activates the traditional 4-pin SPI bus (SS, MOSI, MISO, SCLK)
    dosi.u1.spi.send([0x01, 0x00, 0xC0, 0x08, 0x20])
    # Change I/O Update rate to something rather fast
    dosi.u1.spi.send([0x04, 0x00,0x00,0x00,0x1f])
    # Turn off input clock divider
    dosi.u1.spi.send([0x02,0x1F,0x3F,0xC0,0x00])
 
    print "Welcome to DOSITool"
    import re
    low = 0
    high = 0
    step = 1
    while 1:
        user_in = raw_input("dosi > ")
        if (len(user_in) == 0):
            continue
        elif (user_in[0] == 'q'):
            sys.exit(1)
        elif user_in[0] == 'h':
            print __HELP_STR__
        elif (user_in[0] == 's'):
            if user_in[1] == 'l':
                # clean up frequency input (get rid of trailing white spaces
                i = 0
                try:
                    i = argStringToFTW(user_in[2:])    
                except:
                    print "Proplem with argument"
                    sys.exit(-1)

                low = i
                print("Set low frequency to {0}".format(hex(low)))
            elif user_in[1] == 'h':
                i = 0
                # clean up frequency input (get rid of trailing white spaces
                try:
                    i = argStringToFTW(user_in[2:])    
                except:
                    print "Proplem with argument"
                    sys.exit(-1)

                high = i
                print("Set high frequency to {0}".format(hex(high)))
            elif user_in[1] == 's':
                # clean up frequency input (get rid of trailing white spaces
                i = 0
                try:
                    i = argStringToFTW(user_in[2:])    
                except:
                    print "Proplem with argument"
                    sys.exit(-1)
                step = i
                print("Set step frequency to {0}".format(hex(step)))
            elif user_in[1] == 'f':
                # clean up frequency input (get rid of trailing white spaces
                try:
                    i = argStringToFTW(user_in[2:])    
                except:
                    print "Proplem with argument"
                    sys.exit(-1)
                print "Setting DDS to {0}".format(hex(i))
                dosi.u1.spi.send([0x0E, 0x3f,0xff, 0x00, 0x00,(i>>24)&0xff, (i>>16)&0xff, (i>>8)&0xff, i&0x1000])
        elif (user_in[0] == 'r'):
            for i in range(low,high,step):
                dosi.u1.spi.send([0x0E, 0x3f,0xff, 0x00, 0x00,(i>>24)&0xff, (i>>16)&0xff, (i>>8)&0xff, i&0x1000])
                time.sleep(.01)
